unit DX2LIB;

interface

uses
  Windows;

type
  puint16 = ^uint16;
  puint32 = ^uint32;
  puint64 = ^uint64;

{$IFDEF WIN64}
  TDeviceID   = UInt64;
{$ELSE}
  TDeviceID   = UInt32;
{$ENDIF}


  TErrorCode  = UInt16;
  TInstruction  = uint8;

  // A[Xe[^X
  TAlarmStatus = packed Record
    id      :UInt8;
    Status  :TErrorCode;
  end;
  PAlarmStatus = ^TAlarmStatus;

  TSyncReadParam = packed Record
    addr:   uint16;
    length: uint16;
    ids:    array[0..255] of uint8;
  end;
  PSyncReadParam = ^TSyncReadParam;

  TBulkReadParam = packed Record
    id:     uint8;
    addr:   uint16;
    length: uint16;
  end;
  PBulkReadParam = ^TBulkReadParam;

  TBulkReadResult = packed Record
    size: uint16;
    id:   uint8;
    err:  TErrorCode;
    dat:  array of uint8;
  end;
  PBulkReadResult = ^TBulkReadResult;


  TDX2_OpenPort = function (pcom:pansichar; br:UInt32):TDeviceID; stdcall;
  TDX2_ClosePort = function (dvid:TDeviceID):BOOL; stdcall;
  TDX2_SetBaudrate = function (dvid:TDeviceID; br:UInt32):BOOL; stdcall;
  TDX2_SetTimeOutOffset = procedure (dvid:TDeviceID; offsettime:UInt32); stdcall;

  TDX2_Active = function (dvid:TDeviceID):BOOL; stdcall;

  TDX2_TxPacket = function (dvid:TDeviceID; id:uint8; cInst:TInstruction; var pParam:uint8; iLength:integer; var errcode:TErrorCode):integer; stdcall;
  TDX2_RxPacket = function (dvid:TDeviceID; var pRcv:uint8; sz:integer; var pLength:integer; timeout:DWORD; var errcode:TErrorCode):integer; stdcall;

  TDX2_ReadByteData = function (dvid:TDeviceID; id:uint8; adr:uint16; var result:uint8; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_WriteByteData = function (dvid:TDeviceID; id:uint8; adr:uint16; dat:uint8; var errcode:TErrorCode):BOOL; stdcall;

  TDX2_ReadWordData = function (dvid:TDeviceID; id:uint8; adr:uint16; var result:uint16; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_WriteWordData = function (dvid:TDeviceID; id:uint8; adr:uint16; dat:uint16; var errcode:TErrorCode):BOOL; stdcall;

  TDX2_ReadLongData = function (dvid:TDeviceID; id:uint8; adr:uint16; var result:uint32; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_WriteLongData = function (dvid:TDeviceID; id:uint8; adr:uint16; dat:uint32; var errcode:TErrorCode):BOOL; stdcall;

  TDX2_ReadBlockData = function (dvid:TDeviceID; id:uint8; adr:uint16; var result:uint8; len:integer; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_WriteBlockData = function (dvid:TDeviceID; id:uint8; adr:uint16; var dat:uint8; len:integer; var errcode:TErrorCode):BOOL; stdcall;

  TDX2_Ping = function (dvid:TDeviceID; id:uint8; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_Ping2 = function (dvid:TDeviceID; var num:UInt32; AlarmStatus:PAlarmStatus; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_Reset = function (dvid:TDeviceID; id:uint8; var errcode:TErrorCode):BOOL; stdcall;

  TDX2_ReadSyncData = function (dvid:TDeviceID; param:PSyncReadParam; var num:uint32; var dat:uint8; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_WriteSyncData = function (dvid:TDeviceID; var dat:uint8; size:uint32; var errcode:TErrorCode):BOOL; stdcall;

  TDX2_ReadBulkData = function (dvid:TDeviceID; param:PBulkReadParam; var num:uint32; var dat:uint8; var errcode:TErrorCode):BOOL; stdcall;
  TDX2_WriteBulkData = function (dvid:TDeviceID; var dat:uint8; size:uint32; var errcode:TErrorCode):BOOL; stdcall;

  function DX2LIB_Load:boolean;
  procedure DX2LIB_Unload;
  function DX2LIB_Active:boolean;

const
  INST_PING           = $01;
  INST_READ           = $02;
  INST_WRITE          = $03;
  INST_REG_WRITE      = $04;
  INST_ACTION         = $05;
  INST_RESET          = $06;
  INST_SYNC_WRITE     = $83;
  INST_SYNG_REG_WRITE = $85;


  ERR_INVALID_DEVID   = $8000;
  ERR_INVALID_ID      = $4000;
  ERR_DIFF_ID         = $2000;
  ERR_ILLEGAL_SIZE    = $1000;
  ERR_INVALID_PARAM   = $0800;
  ERR_COMM            = $0400;
  ERR_CHECKSUM        = $0200;
  ERR_TIMEOUT         = $0100;
  ERR_DX2_ALERT       = $80;

  ERR_DX2_ACCESS      = 7;
  ERR_DX2_DATALIMIT   = 6;
  ERR_DX2_DATALENGTH  = 5;
  ERR_DX2_DATARANGE   = 4;
  ERR_DX2_CRC         = 3;
  ERR_DX2_INSTRUCTION = 2;
  ERR_DX2_RESULT      = 1;
  ERR_NON             = 0;

var
  DX2_OpenPort        :TDX2_OpenPort;
  DX2_ClosePort       :TDX2_ClosePort;
  DX2_SetBaudrate     :TDX2_SetBaudrate;
  DX2_SetTimeOutOffset:TDX2_SetTimeOutOffset;
  DX2_Active          :TDX2_Active;

  DX2_TxPacket        :TDX2_TxPacket;
  DX2_RxPacket        :TDX2_RxPacket;

  DX2_ReadByteData    :TDX2_ReadByteData;
  DX2_WriteByteData   :TDX2_WriteByteData;
  DX2_ReadWordData    :TDX2_ReadWordData;
  DX2_WriteWordData   :TDX2_WriteWordData;
  DX2_ReadLongData    :TDX2_ReadLongData;
  DX2_WriteLongData   :TDX2_WriteLongData;

  DX2_ReadBlockData   :TDX2_ReadBlockData;
  DX2_WriteBlockData  :TDX2_WriteBlockData;

  DX2_Ping            :TDX2_Ping;
  DX2_Ping2           :TDX2_Ping2;
  DX2_Reset           :TDX2_Reset;

  DX2_ReadSyncData    :TDX2_ReadSyncData;
  DX2_WriteSyncData   :TDX2_WriteSyncData;
  DX2_ReadBulkData    :TDX2_ReadBulkData;
  DX2_WriteBulkData   :TDX2_WriteBulkData;

implementation

var
  DLLHandle:THANDLE;

function conv_s11(v:puint16):integer;
begin
  if v^ and $400 <> 0 then result := -(v^ and $3ff) else result := (v^ and $3ff);
end;

function conv_sig(v:puint16):integer;
begin
  result := int16(v^);
end;

function conv_wd(v:puint16):integer;
begin
  result := uint16(v^);
end;

function DX2LIB_Load:boolean;
var
  fn:string;
begin
{$IFDEF WIN64}
  DLLHandle := LoadLibrary('dx2lib_x64.dll');
{$ELSE}
  DLLHandle := LoadLibrary('dx2lib_x32.dll');
{$ENDIF}
  if DLLHandle <> 0 then
  begin
    fn :='DX2_OpenPort';
    @DX2_OpenPort := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_ClosePort';
    @DX2_ClosePort := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_SetBaudrate';
    @DX2_SetBaudrate := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_SetTimeOutOffset';
    @DX2_SetTimeOutOffset := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_Active';
    @DX2_Active := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_TxPacket';
    @DX2_TxPacket := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_RxPacket';
    @DX2_RxPacket := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_ReadByteData';
    @DX2_ReadByteData := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_WriteByteData';
    @DX2_WriteByteData := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_ReadWordData';
    @DX2_ReadWordData := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_WriteWordData';
    @DX2_WriteWordData := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_ReadLongData';
    @DX2_ReadLongData := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_WriteLongData';
    @DX2_WriteLongData := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_ReadBlockData';
    @DX2_ReadBlockData := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_WriteBlockData';
    @DX2_WriteBlockData := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_Ping';
    @DX2_Ping := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_Ping2';
    @DX2_Ping2 := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_Reset';
    @DX2_Reset := GetProcAddress(DLLHandle, PChar(fn));

    fn :='DX2_ReadSyncData';
    @DX2_ReadSyncData := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_WriteSyncData';
    @DX2_WriteSyncData := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_ReadBulkData';
    @DX2_ReadBulkData := GetProcAddress(DLLHandle, PChar(fn));
    fn :='DX2_WriteBulkData';
    @DX2_WriteBulkData := GetProcAddress(DLLHandle, PChar(fn));
  end;

  result :=
    (DLLHandle <> 0)
    and (@DX2_OpenPort <> nil)
    and (@DX2_ClosePort <> nil)
    and (@DX2_SetBaudrate <> nil)
    and (@DX2_SetTimeOutOffset <> nil)
    and (@DX2_Active <> nil)
    and (@DX2_TxPacket <> nil)
    and (@DX2_RxPacket <> nil)
    and (@DX2_ReadByteData <> nil)
    and (@DX2_WriteByteData <> nil)
    and (@DX2_ReadWordData <> nil)
    and (@DX2_WriteWordData <> nil)
    and (@DX2_ReadLongData <> nil)
    and (@DX2_WriteLongData <> nil)
    and (@DX2_ReadBlockData <> nil)
    and (@DX2_WriteBlockData <> nil)
    and (@DX2_Ping <> nil)
    and (@DX2_Ping2 <> nil)
    and (@DX2_Reset <> nil)
    and (@DX2_ReadSyncData <> nil)
    and (@DX2_WriteSyncData <> nil)
    and (@DX2_ReadBulkData <> nil)
    and (@DX2_WriteBulkData <> nil);
end;

procedure DX2LIB_Unload;
begin
  if DX2LIB_Active then
  begin
    FreeLibrary(DLLHandle);
    DX2_OpenPort := nil;
    DX2_ClosePort := nil;
    DX2_SetBaudrate := nil;
    DX2_SetTimeOutOffset := nil;
    DX2_Active := nil;
    DX2_TxPacket := nil;
    DX2_RxPacket := nil;
    DX2_ReadByteData := nil;
    DX2_WriteByteData := nil;
    DX2_ReadWordData := nil;
    DX2_WriteWordData := nil;
    DX2_ReadLongData := nil;
    DX2_WriteLongData := nil;
    DX2_ReadBlockData := nil;
    DX2_WriteBlockData := nil;
    DX2_Ping := nil;
    DX2_Ping2 := nil;
    DX2_Reset := nil;
    DX2_ReadSyncData := nil;
    DX2_WriteSyncData := nil;
    DX2_ReadBulkData := nil;
    DX2_WriteBulkData := nil;
  end;
end;

function DX2LIB_Active:boolean;
begin
  result := DLLHandle <> 0;
end;

begin
  DLLHandle := 0;
end.

