/*----------------------------------------------------------*
    dx2lib.h
    Dynamixel2プロトコル用API V1.3
                                       Last Edit '16 11/09
   Copyright (c) 2005, 2016 BestTechnology CO.,LTD.
 *----------------------------------------------------------*/

#ifndef _DX2LIB_H_INCLUDE
#define _DX2LIB_H_INCLUDE

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
  #include <windows.h>
#else
  #include <pthread.h>
#endif

#ifndef _MSC_VER
  #include <stdbool.h>
#endif

#include <stdint.h>

#define BROADCASTING_ID     (0xfe)

#define DXMAXLENGTH         (10000)

#ifndef _WIN32
  typedef int                             HANDLE;
  typedef pthread_mutex_t                 CRITICAL_SECTION;

  #define EnterCriticalSection(mtx)       pthread_mutex_lock(mtx)
  #define LeaveCriticalSection(mtx)       pthread_mutex_unlock(mtx)
  #define DeleteCriticalSection(mtx)      pthread_mutex_destroy(mtx)
  #define InitializeCriticalSection(mtx)  pthread_mutex_init(mtx,NULL)
  #define SetEvent(evt)                   pthread_cond_signal(evt)
  #define INVALID_int_VALUE               -1
  #define INVALID_HANDLE_VALUE            -1
#endif

// Device ID
#ifdef _WIN32
  #ifdef _WIN64
    typedef uint64_t TDeviceID;
  #else
    typedef uint32_t TDeviceID;
  #endif
#else
  #ifdef __x86_64__
    typedef uint64_t TDeviceID;
  #else
    typedef uint32_t TDeviceID;
  #endif
#endif

// Kind of instruction
typedef enum {INST_PING = 1, INST_READ = 2, INST_WRITE = 3, INST_REG_WRITE = 4, INST_ACTION = 5, INST_RESET = 6, INST_REBOOT = 8, INST_STATUS = 0x55, INST_SYNC_READ = 0x82, INST_SYNC_WRITE = 0x83, INST_BULK_READ = 0x92, INST_BULK_WRITE = 0x93} TInstruction;

// Error flags
typedef uint16_t TErrorCode;
#define ERR_INVALID_DEVID     (1 << 15)
#define ERR_INVALID_ID        (1 << 14)
#define ERR_DIFF_ID           (1 << 13)
#define ERR_ILLEGAL_SIZE      (1 << 12)
#define ERR_INVALID_PARAM     (1 << 11)
#define ERR_COMM              (1 << 10)
#define ERR_CHECKSUM          (1 << 9)
#define ERR_TIMEOUT           (1 << 8)
#define ERR_DX2_ALERT         (1 << 7)
#define ERR_DX2_ACCESS        (7)
#define ERR_DX2_DATALIMIT     (6)
#define ERR_DX2_DATALENGTH    (5)
#define ERR_DX2_DATARANGE     (4)
#define ERR_DX2_CRC           (3)
#define ERR_DX2_INSTRUCTION   (2)
#define ERR_DX2_RESULT        (1)
#define ERR_NON               (0)

// Alarm status
#ifdef _MSC_VER
__pragma (pack (push, 1))

typedef struct {
  uint8_t     id;
  TErrorCode  Status;
} TDx2AlarmStatus;

typedef struct {
  uint16_t addr;
  uint16_t length;
  uint8_t  ids[256];
} TSyncReadParam;

typedef struct {
  uint8_t  id;
  uint16_t addr;
  uint16_t length;
} TBulkReadParam;

typedef struct {
  uint16_t    size;
  uint8_t     id;
  TErrorCode  err;
  uint8_t     dat[];
} TBulkReadResult, *PBulkReadResult;

__pragma (pack (pop))
#elif defined(__GNUC__)
typedef struct {
  uint8_t     id;
  TErrorCode  Status;
} __attribute__ ((gcc_struct, __packed__)) TDx2AlarmStatus, *PDx2AlarmStatus;

typedef struct {
  uint16_t addr;
  uint16_t length;
  uint8_t  ids[256];
} __attribute__ ((gcc_struct, __packed__)) TSyncReadParam;

typedef struct {
  uint8_t  id;
  uint16_t addr;
  uint16_t length;
} __attribute__ ((gcc_struct, __packed__)) TBulkReadParam;

typedef struct {
  uint16_t    size;
  uint8_t     id;
  TErrorCode  err;
  uint8_t     dat[];
} __attribute__ ((gcc_struct, __packed__)) TBulkReadResult, *PBulkReadResult;
#endif

#ifdef _WIN32
  #ifdef __cplusplus
    #ifdef __MAKE_LIB__
      #define DXAPIDLL extern "C" __declspec(dllexport)
    #else
      #define DXAPIDLL extern "C" __declspec(dllimport)
    #endif
  #else
    #ifdef __MAKE_LIB__
      #define DXAPIDLL __declspec(dllexport)
    #else
      #define DXAPIDLL __declspec(dllimport)
    #endif
  #endif
#else
  #define DXAPIDLL
  #define WINAPI
#endif

#ifndef _DYNAMICLOAD
// When static DLL calls is used the following prototype declaration
DXAPIDLL TDeviceID WINAPI DX2_OpenPort (char *name, uint32_t baud);
DXAPIDLL bool WINAPI DX2_ClosePort (TDeviceID dvid);
DXAPIDLL bool WINAPI DX2_SetBaudrate (TDeviceID dvid, uint32_t baud);
DXAPIDLL bool WINAPI DX2_Active (TDeviceID dvid);
DXAPIDLL void WINAPI DX2_SetTimeOutOffset (TDeviceID dvid, uint32_t t);
DXAPIDLL double WINAPI GetQueryPerformanceCounter (void);
DXAPIDLL bool WINAPI DX2_TxPacket (TDeviceID dvid, uint8_t id, TInstruction inst, const uint8_t *param, uint32_t len, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_RxPacket (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_ReadByteData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_WriteByteData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_ReadWordData (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdata, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_WriteWordData (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_ReadLongData (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t *rdata, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_WriteLongData (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_ReadBlockData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, uint32_t len, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_WriteBlockData (TDeviceID dvid, uint8_t id, uint16_t adr, const uint8_t *dat, uint32_t len, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_Ping (TDeviceID dvid, uint8_t id, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_Ping2 (TDeviceID dvid, uint32_t *num, TDx2AlarmStatus *AlarmStatus, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_ReadSyncData (TDeviceID dvid, const TSyncReadParam *param, uint32_t *num, uint8_t *dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_WriteSyncData (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_ReadBulkData (TDeviceID dvid, const TBulkReadParam *param, uint32_t *num, uint8_t *dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_WriteBulkData (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_Reset (TDeviceID dvid, uint8_t id, TErrorCode *err);
DXAPIDLL bool WINAPI DX2_Reboot (TDeviceID dvid, uint8_t id, TErrorCode *err);

#else
// When the dynamic DLL calls is used the following prototype declaration
TDeviceID (WINAPI *DX2_OpenPort) (char *name, uint32_t baud) = NULL;
bool (WINAPI *DX2_ClosePort) (TDeviceID dvid) = NULL;
bool (WINAPI *DX2_SetBaudrate) (TDeviceID dvid, uint32_t baud) = NULL;
bool (WINAPI *DX2_Active) (TDeviceID dvid) = NULL;
void (WINAPI *DX2_SetTimeOutOffset) (TDeviceID dvid, uint32_t offsettime) = NULL;
double (WINAPI *GetQueryPerformanceCounter) (void) = NULL;
bool (WINAPI *DX2_TxPacket) (TDeviceID dvid, uint8_t id, TInstruction inst, uint8_t *param, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *DX2_RxPacket) (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err) = NULL;
bool (WINAPI *DX2_ReadByteData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, TErrorCode *err) = NULL;
bool (WINAPI *DX2_WriteByteData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *err) = NULL;
bool (WINAPI *DX2_ReadWordData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdata, TErrorCode *err) = NULL;
bool (WINAPI *DX2_WriteWordData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *err) = NULL;
bool (WINAPI *DX2_ReadLongData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t *rdata, TErrorCode *err) = NULL;
bool (WINAPI *DX2_WriteLongData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t dat, TErrorCode *err) = NULL;
bool (WINAPI *DX2_ReadBlockData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *DX2_WriteBlockData) (TDeviceID dvid, uint8_t id, uint16_t adr, const uint8_t *dat, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *DX2_Ping) (TDeviceID dvid, uint8_t id, TErrorCode *err) = NULL;
bool (WINAPI *DX2_Ping2) (TDeviceID dvid, uint32_t *num, TDx2AlarmStatus *AlarmStatus, TErrorCode *err) = NULL;
bool (WINAPI *DX2_ReadSyncData) (TDeviceID dvid, const TSyncReadParam *param, uint32_t *num, uint8_t *dat, TErrorCode *err) = NULL;
bool (WINAPI *DX2_WriteSyncData) (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err) = NULL;
bool (WINAPI *DX2_ReadBulkData) (TDeviceID dvid, const TBulkReadParam *param, uint32_t *num, uint8_t *dat, TErrorCode *err) = NULL;
bool (WINAPI *DX2_WriteBulkData) (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err) = NULL;
bool (WINAPI *DX2_Reset) (TDeviceID dvid, uint8_t id, TErrorCode *err) = NULL;
bool (WINAPI *DX2_Reboot) (TDeviceID dvid, uint8_t id, TErrorCode *err) = NULL;

static HMODULE hModule = NULL;

#ifdef _WIN32
  #ifdef _WIN64
  #define DLLNAME   "dx2lib_x64.dll"
  #else
  #define DLLNAME   "dx2lib_x32.dll"
  #endif
#else
  #ifdef __x86_64__
  #define DLLNAME   "dx2lib_x64.dll"
  #else
  #define DLLNAME   "dx2lib_x32.dll"
  #endif
#endif

// Load the DLL
bool LoadDLL (void) {
  hModule = LoadLibrary (DLLNAME);
  if (hModule == NULL) {
    return false;
  }

  DX2_OpenPort         = (void *)GetProcAddress (hModule, "DX2_OpenPort");
  DX2_ClosePort        = (void *)GetProcAddress (hModule, "DX2_ClosePort");
  DX2_SetBaudrate      = (void *)GetProcAddress (hModule, "DX2_SetBaudrate");
  DX2_Active           = (void *)GetProcAddress (hModule, "DX2_Active");
  DX2_SetTimeOutOffset = (void *)GetProcAddress (hModule, "DX2_SetTimeOutOffset");
  GetQueryPerformanceCounter = (void *)GetProcAddress (hModule, "GetQueryPerformanceCounter");
  DX2_TxPacket         = (void *)GetProcAddress (hModule, "DX2_TxPacket");
  DX2_RxPacket         = (void *)GetProcAddress (hModule, "DX2_RxPacket");
  DX2_ReadByteData     = (void *)GetProcAddress (hModule, "DX2_ReadByteData");
  DX2_WriteByteData    = (void *)GetProcAddress (hModule, "DX2_WriteByteData");
  DX2_ReadWordData     = (void *)GetProcAddress (hModule, "DX2_ReadWordData");
  DX2_WriteWordData    = (void *)GetProcAddress (hModule, "DX2_WriteWordData");
  DX2_ReadLongData     = (void *)GetProcAddress (hModule, "DX2_ReadLongData");
  DX2_WriteLongData    = (void *)GetProcAddress (hModule, "DX2_WriteLongData");
  DX2_ReadBlockData    = (void *)GetProcAddress (hModule, "DX2_ReadBlockData");
  DX2_WriteBlockData   = (void *)GetProcAddress (hModule, "DX2_WriteBlockData");
  DX2_ReadSyncData     = (void *)GetProcAddress (hModule, "DX2_ReadSyncData");
  DX2_WriteSyncData    = (void *)GetProcAddress (hModule, "DX2_WriteSyncData");
  DX2_ReadBulkData     = (void *)GetProcAddress (hModule, "DX2_ReadBulkData");
  DX2_WriteBulkData    = (void *)GetProcAddress (hModule, "DX2_WriteBulkData");
  DX2_Ping             = (void *)GetProcAddress (hModule, "DX2_Ping");
  DX2_Ping2            = (void *)GetProcAddress (hModule, "DX2_Ping2");
  DX2_Reset            = (void *)GetProcAddress (hModule, "DX2_Reset");
  DX2_Reboot           = (void *)GetProcAddress (hModule, "DX2_Reboot");

  return
    (DX2_OpenPort         != NULL) &&
    (DX2_ClosePort        != NULL) &&
    (DX2_SetBaudrate      != NULL) &&
    (DX2_Active           != NULL) &&
    (DX2_SetTimeOutOffset != NULL) &&
    (GetQueryPerformanceCounter != NULL) &&
    (DX2_TxPacket         != NULL) &&
    (DX2_RxPacket         != NULL) &&
    (DX2_ReadByteData     != NULL) &&
    (DX2_WriteByteData    != NULL) &&
    (DX2_ReadWordData     != NULL) &&
    (DX2_WriteWordData    != NULL) &&
    (DX2_ReadLongData     != NULL) &&
    (DX2_WriteLongData    != NULL) &&
    (DX2_ReadBlockData    != NULL) &&
    (DX2_WriteBlockData   != NULL) &&
    (DX2_ReadSyncData     != NULL) &&
    (DX2_WriteSyncData    != NULL) &&
    (DX2_ReadBulkData     != NULL) &&
    (DX2_WriteBulkData    != NULL) &&
    (DX2_Ping             != NULL) &&
    (DX2_Ping2            != NULL) &&
    (DX2_Reset            != NULL) &&
    (DX2_Reset            != NULL);
}

// Unload the DLL
void UnloadDLL (void) {
  if (hModule != NULL) {
    FreeLibrary (hModule);
    DX2_OpenPort         = NULL;
    DX2_ClosePort        = NULL;
    DX2_SetBaudrate      = NULL;
    DX2_Active           = NULL;
    DX2_SetTimeOutOffset = NULL;
    GetQueryPerformanceCounter = NULL;
    DX2_TxPacket         = NULL;
    DX2_RxPacket         = NULL;
    DX2_ReadByteData     = NULL;
    DX2_WriteByteData    = NULL;
    DX2_ReadWordData     = NULL;
    DX2_WriteWordData    = NULL;
    DX2_ReadLongData     = NULL;
    DX2_WriteLongData    = NULL;
    DX2_ReadBlockData    = NULL;
    DX2_WriteBlockData   = NULL;
    DX2_ReadSyncData     = NULL;
    DX2_WriteSyncData    = NULL;
    DX2_ReadBulkData     = NULL;
    DX2_WriteBulkData    = NULL;
    DX2_Ping             = NULL;
    DX2_Ping2            = NULL;
    DX2_Reset            = NULL;
    DX2_Reboot           = NULL;
  }
}

#endif

#ifdef __cplusplus
}
#endif

#endif //_DX2LIB2_H_INCLUDE
